//+------------------------------------------------------------------+
//|                                                SuperSmoother.mq4 |
//| Algorithm taken from                                             |
//| "Cycle Analytics for Traders"                                    |
//| by John F. Ehlers, 2013                                          |
//|                                         Copyright  2014, Kirill |
//|                                          StockProgrammer@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, Kirill"
#property link      ""

//+------------------------------------------------------------------+
//| Description                                                      |
//|                                                                  |
//| SuperSmoother =                                                  |
//|      = 2-Pole Modified Butterworth Filter applied to SMA(2)      |
//| Recommended for universal use to remove aliasing noise           |
//| Best used with AttenuationPeriod = 10                            |
//| Use higher AttenuationPeriod if necessary, lower is not advised  |
//| Feed output data into other indicators for analysis if required  |
//| Better than using raw price data due to aliasing noise           |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Lime
//---- indicator parameters
extern int AttenuationPeriod=10;                //Note: this is NOT the amount of 
                                                //bars used in the calculation. This 
                                                //is the cyclic period, signal 
                                                //components below which are attenuated
extern int Shift = 0;                           //Shift Left
//---- indicator buffers
double ExtMapBuffer[];
//----
int ExtCountedBars=0;
int BarsUsed = 3;                               //This indicator always needs data from 
                                                //the current bar + only 2 previous bars
//---- declare constants
   double a,b,c1,c2,c3,d3;
   double pi =    3.1415926535;
   double sqrt2 = 1.4142135623;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){
   int    draw_begin;
   string short_name;   
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,3);
   SetIndexShift(0,-1*Shift);   
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   if(AttenuationPeriod<2) AttenuationPeriod=2; //Maximum possible frequency (without aliasing) 
                                                //is Nyquist Frequency = 0.5 * Sampling Frequency 
                                                //= 0.5 * 1 Cycle per Bar = 0.5 Cycles per Sample
                                                //I.e. the Sampling Frequecny should be 
                                                //>= 2 * Filter Frequency. 
                                                //Therefore, Filter Frequency 
                                                //<= 0.5 * Sampling Frequency
                                                //Therefore, the minimum Period is 1/F = 2 bars
   SetIndexDrawBegin(0,BarsUsed);                
//---- indicator short name
   IndicatorShortName("SuperSmoother("+AttenuationPeriod+")");
//---- indicator buffers mapping
   SetIndexBuffer(0,ExtMapBuffer);
//---- initialize constants
   a  = MathExp(-1 * sqrt2 * pi / AttenuationPeriod);
   b  = 2 * a * MathCos(sqrt2 * pi / AttenuationPeriod);
   c2 = b;  
   c3 = -1 * a * a;
   c1 = 1 - c2 - c3;   
//---- initialization done
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start(){
   if(Bars<=BarsUsed) return(0);
   ExtCountedBars=IndicatorCounted();
//---- check for possible errors
   if (ExtCountedBars<0) return(-1);
//---- last counted bar will be recounted
   if (ExtCountedBars>0) ExtCountedBars--;
//----
   SuperSmoother();
//---- done
   return(0);
}
//+------------------------------------------------------------------+
//| SuperSmoother Calculation                                        |
//+------------------------------------------------------------------+
void SuperSmoother(){
   int i,pos = Bars-ExtCountedBars-1;
//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<BarsUsed;i++){
         ExtMapBuffer[Bars-i]=0; 
         pos--;
      }
//---- calculate all other bars      
   while(pos>=0){
      ExtMapBuffer[pos] = c1 * (Close[pos] + Close[pos+1]) / 2 + c2 * ExtMapBuffer[pos+1] + c3 * ExtMapBuffer[pos+2];
 	   pos--;
   }
}

